from typing import List

from .constants import SUPPORTED_ITEM_TYPES
from .models import Result
from .utils import default_checker


def binary_search(term: int, items: List[int]):
    raise NotImplementedError


def simple_search(term: SUPPORTED_ITEM_TYPES, items: List[SUPPORTED_ITEM_TYPES]):
    results = []

    for index in range(len(items)):
        item = items[index]
        match = default_checker(item, term)
        if match[0]:
            results.append(Result(index=index, value=item, confidence=match[1]))

    return results


def search_for_least_frequent_items(size: int, items: List[SUPPORTED_ITEM_TYPES]):
    raise NotImplementedError


def search_for_most_frequent_items(size: int, items: List[SUPPORTED_ITEM_TYPES]):
    raise NotImplementedError


def search_csv_file(filename: str, column: str, value: str):
    raise NotImplementedError



from setuptools import setup, find_packages

VERSION = '0.0.4' 
DESCRIPTION = "Find what you're looking for in a flash with Huntela - the ultimate search tool for Python."
LONG_DESCRIPTION = "Huntela is the ultimate search tool for Python. With a variety of powerful algorithms to choose from, finding what you're looking for has never been easier. From binary search to linear search and more, Huntela has everything you need to quickly and efficiently search through your data. With a simple, intuitive interface and lightning-fast performance, Huntela is the go-to package for anyone who needs to search through large amounts of data. Whether you're a data scientist, engineer, or developer, Huntela will help you find what you need in no time."

# Setting up
setup(
    # the name must match the folder name 'verysimplemodule'
    name="huntela", 
    version=VERSION,
    author="Tomisin Abiodun",
    author_email="decave.12357@gmail.com",
    description=DESCRIPTION,
    long_description=LONG_DESCRIPTION,
    packages=find_packages(),
    install_requires=[], # add any additional packages that 
    # needs to be installed along with your package. Eg: 'caer'

    keywords=['python', 'first package'],
    classifiers= [
        "Development Status :: 3 - Alpha",
        "Intended Audience :: Education",
        "Programming Language :: Python :: 2",
        "Programming Language :: Python :: 3",
        "Operating System :: MacOS :: MacOS X",
        "Operating System :: Microsoft :: Windows",
    ]
)



from collections import Counter
import itertools

from .constants import SUPPORTED_ITEM_TYPES


def default_checker(item_1: SUPPORTED_ITEM_TYPES, item_2: SUPPORTED_ITEM_TYPES):
    if item_1 == item_2:
        return (True, 1)

    if type(item_1) is str and type(item_2) is str and (len(item_1) > 1 or len(item_2) > 1):
        item_1_char_count = char_count(item_1)
        item_2_char_count = char_count(item_2)

        combined_chars = set()

        unconsumed_chars = 0
        total_chars = 0

        for char in item_1_char_count:
            combined_chars.add(char)
        for char in item_2_char_count:
            combined_chars.add(char)

        for char in combined_chars:
            count_in_item_1 = item_1_char_count[char] if char in item_1_char_count else 0
            count_in_item_2 = item_2_char_count[char] if char in item_2_char_count else 0

            combined_count = count_in_item_1 + count_in_item_2

            unconsumed_chars += abs(count_in_item_1 - count_in_item_2)
            total_chars += combined_count

        percentage_match = 1 - (unconsumed_chars / max(len(item_1), len(item_2)))
        if percentage_match > 0.5:
            return (True, round(percentage_match, 1))

    return (False, None)


def char_count(s):
    """
    Returns a dictionary mapping each character in a string to the number of times it appears.

    Args:
        s (str): Input string.

    Returns:
        dict: Dictionary mapping each character to the number of times it appears.
    """

    char_dict = {}
    for char, count in Counter(itertools.chain.from_iterable(s)).items():
        char_dict[char] = count

    return char_dict
